%{
/* JML - Java Markup Language
 * Copyright (C) 1998 Scott Dybiec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


import java.lang.Math;
import java.io.*;
import java.util.StringTokenizer;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import com.sun.java.swing.text.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.preview.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.Hashtable;
%}
       
/* YACC Declarations */
%token NUMBER NL OPERATOR PAREN CARET
%token QSTRING BADSTRING COMMA ID
%token OBRACE CBRACE
%token FORM TITLE
%token HORIZONTALBOX VERTICALBOX
%token HOMOGENEOUS SPACING
%token FRAME
%token BORDERWIDTH
%token BUTTON TOGGLEBUTTON CHECKBUTTON
%token LABEL
%token WINDOW
%token TOOLTIP
%token ENTRY
%token NOTEBOOK PAGE
%token TABPOSITION TOP BOTTOM LEFT RIGHT
%token RADIOBUTTON RADIOGROUP
%token LIST LISTITEM
%token FRAMESHADOW SHADOW_IN SHADOW_OUT SHADOW_NONE
%token SHADOW_ETCHED_IN SHADOW_ETCHED_OUT
%token WIDGET
%token ONCLICK ONPRESS ONRELEASE ONENTER ONLEAVE ONTOGGLE
%token SENSITIZE INSENSITIZE SHOW HIDE
%token TOGGLESENSITIVITY TOGGLEVISIBILITY
%token SUBMITFORM RESETFORM CANCELFORM
%token JUSTIFY CENTER FILL
%token FILECHOOSER DIRCHOOSER
%token TABLE TABLEDATUM
%token XPADDING YPADDING
%token COMBOBOX COMBOITEM
%token TRUE FALSE

/* Grammar follows */
%%

widgetseries: widget
             { if ( ($1.obj != null) && (!containerStack.empty()) )
               {
                 Component widget = (Component)$1.obj;
                 System.out.println("Found widgetseries");
                 widget.setVisible(true);
                 
                 Class c = $1.obj.getClass();
                 String s = c.getName();
                 System.out.println("Adding component "+s);
                 Container container = (Container)containerStack.peek();
                 container.add(widget); 
               }
             }

        | widgetseries widget
             { if ( ($2.obj != null) && (!containerStack.empty()) )
               {
                 Component widget = (Component)$2.obj;
                 System.out.println("Found widgetseries widget");
                 widget.setVisible(true);
                 
                 Class c = $2.obj.getClass();
                 String s = c.getName();
                 System.out.println("Adding component "+s);
                 Container container = (Container)containerStack.peek();
                 container.add(widget); 
               }
             }
        ;

widgets: /* empty */
	| widgetseries
	;

widget: boxblock
	| frameblock
	| labelblock
	| entryblock
	| buttonblock
	| notebookblock
	| radiogroupblock
	| listblock
	| windowblock     
	| widgetinstance
	| filechooser
	| dirchooser
	| tableblock
	| comboboxblock
	;

windowblock: window optid OBRACE title borderwidth widgets CBRACE
                     { 
                       JBorderFrame frame = (JBorderFrame)$1.obj;
                       System.out.println("Found windowblock");
                       frame.setTitle($4.sval);
                       if ($4 != null)
                         frame.setBorderSize($5.ival);
                       containerStack.pop();
                       frame.pack();
                       frame.setVisible(true);
                       $$.obj = frame;
                     }
	;

window: WINDOW       { 
                       JBorderFrame frame = new JBorderFrame();
                       frame.addWindowListener(new WindowAdapter() 
                           {
                             public void windowClosing(WindowEvent e)
                             { 
                               System.exit(0);
                             }
                           });
                       frame.getContentPane().setLayout(new BorderLayout());
                       //frame.setSize(400, 350);
                       containerStack.push(frame.getContentPane());
                       $$.obj = frame;
                     }
	;

optid: /* empty */ { $$ = null; }
	| identifier
	;

identifier: ID     { 
                      Component c = (Component)containerStack.peek();
                      if (instanceTable.get($1.sval) == null)
                         instanceTable.put($1.sval, c);
                      else
                         System.out.println("Duplicate identifier " + $1.sval);
                      $$.sval = $1.sval; 
                   }

id:     ID         { $$.sval = $1.sval; }
	| QSTRING  { 
                     System.out.println(
                      "Error: Unquoted string required, but quoted found.");
                     $$ = $1;  /* use it anyway */
                   }

qstring: QSTRING   { $$.sval = $1.sval; }

title: /* empty */      { $$ = null; }
	| TITLE qstring	{ $$.sval = $2.sval; }
	;

widgetinstance: WIDGET ID  { 
                             System.out.println("Found widgetinstance "+$2.sval);
                             Component widget;
                             widget = (Component)instanceTable.get($2.sval);
                             if (widget == null) {
                                System.out.println("Widget not found "+$2.sval);
                                $$.obj = null;
                             }
                             else
                                $$.obj = widget;
                           }

boxblock:  box optid OBRACE spacing borderwidth justify widgets CBRACE
               {
                 int gap;
                 Orientation o;
                 int justify;
                 JPanel box = (JPanel)$1.obj; 
                 System.out.println("borderwidth = " + $5.ival);
                 if ($5.ival != 0) {
                   Border emptyBorder = 
                     new EmptyBorder($5.ival,$5.ival,$5.ival,$5.ival);
                   box.setBorder(emptyBorder);
                 }
                 Class c = box.getLayout().getClass();
                 String s = c.getName();
                 System.out.println("Box Layout type "+s);

                 if ($4 != null)
                    gap = $4.ival;
                 else
                    gap = 0;

                 if ($6 != null)
                    o = (Orientation)$6.obj;
                 else 
                    o = Orientation.CENTER;

                 if (s.equals("RowLayout"))
                    box.setLayout(new RowLayout(o, Orientation.FILL, gap));
                 else
                    box.setLayout(new ColumnLayout(Orientation.FILL, o, gap));
                 box.validate();
                 containerStack.pop();
               }

box: HORIZONTALBOX       { 
                           JPanel hbox = new JPanel();
                           LayoutManager mgr = new RowLayout(0); 
                           hbox.setLayout(mgr);
                           containerStack.push(hbox);
                           hbox.setOpaque(false);
                           System.out.println("Created HorizontalBox");
                           $$.obj = hbox;
                         }
	| VERTICALBOX    {
                           JPanel vbox = new JPanel();
                           LayoutManager mgr = new ColumnLayout(0);
                           vbox.setLayout(mgr);
                           containerStack.push(vbox);
                           vbox.setOpaque(false);
                           System.out.println("Created VerticalBox");
                           $$.obj = vbox;
                          }

spacing: /* empty */      { $$ = null; }
	| SPACING NUMBER  { $$.ival = $2.ival; }
	;

justify: /* empty */        { $$ = null; }
	| JUSTIFY position  { $$.obj = $2.obj; }

position: TOP     { $$.obj = Orientation.TOP; }
	| BOTTOM  { $$.obj = Orientation.BOTTOM; }
	| LEFT    { $$.obj = Orientation.LEFT; }
	| RIGHT   { $$.obj = Orientation.RIGHT; }
	| CENTER  { $$.obj = Orientation.CENTER; }
	| FILL    { $$.obj = Orientation.FILL; }

frameblock: frame optid OBRACE title widgets CBRACE
                { 
                  JPanel titledPanel = (JPanel)$1.obj;
                  Border titledBorder = new TitledBorder(null, $4.sval,
                                            TitledBorder.LEFT, TitledBorder.TOP,
                                            null);
                  Border emptyBorder = new EmptyBorder(15,15,15,15);
                  Border compoundBorder = new CompoundBorder( titledBorder, 
                                                              emptyBorder);
                  //titledPanel.setBorder(compoundBorder);
                  titledPanel.setBorder(titledBorder);

                  containerStack.pop();
                }

frame: FRAME       { 
                     JPanel titledPanel = new JPanel();
                     titledPanel.setLayout(new RowLayout(Orientation.FILL, Orientation.FILL, 0));
                     containerStack.push(titledPanel);
                     $$.obj = titledPanel;
                   }

borderwidth: /* empty */      { $$ = new jmlparserval(0);; }
	| BORDERWIDTH NUMBER  { $$.ival = $2.ival; }

labelblock: label optid qstring   
                            { 
                              JLabel label = (JLabel)containerStack.pop();
                              label.setFont(boldFont);
                              label.setText($3.sval);
                              $$.obj = label;
                            }

label: LABEL       { 
                     JLabel label = new JLabel();
                     label.setFont(boldFont);
                     containerStack.push(label);
                   }

entryblock: entry optid maxlength  
                        {
                          JTextField tf = (JTextField)containerStack.pop();
                          tf.setColumns($3.ival);
                          $$.obj = tf;
                        }

entry: ENTRY  { 
                JTextField tf = new JTextField("");
                //JLabel l = new JLabel("TF Label");
                //l.setLabelFor(tf);
                containerStack.push(tf);
              }

maxlength: /* empty */  { $$ = new jmlparserval(10); }
	| NUMBER        { $$.ival = $1.ival; }

buttonblock: simplebutton
	| complexbutton

simplebutton: button optid qstring   
                               {
                                 AbstractButton b = (AbstractButton)$1.obj;
                                 //b.setMargin(new Insets(2,4,2,4));
                                 b.setText($3.sval);
                                 containerStack.pop();
                                 $$.obj = $1.obj;
                               }

complexbutton: button optid OBRACE borderwidth tooltip signals widgets CBRACE  
                    { 
                      System.out.println("Found complex buttonblock");
                      JComponent button = (JComponent)$1.obj;
                      if ($5 != null)
                        button.setToolTipText($5.sval);
                      containerStack.pop();
                      $$.obj = $1.obj;
                    }

button: BUTTON              { 
                              JButton button = new JButton();
                              button.setMargin(new Insets(0,0,0,0));
                              containerStack.push(button);
                              $$.obj = button;
                            }
	| TOGGLEBUTTON      { 
                              JToggleButton button = new JToggleButton();
                              button.setMargin(new Insets(0,0,0,0));
                              button.setLayout(new RowLayout(Orientation.CENTER,
                                                      Orientation.CENTER, 5));
                              containerStack.push(button);
                              $$.obj = button;
                            }
	| CHECKBUTTON       {
                              JCheckBox button = new JCheckBox();
                              button.setMargin(new Insets(0,14,0,0));
                              containerStack.push(button);
                              $$.obj = button;
                            }

signals: /* empty */     
	| signalset     
	;

signalset: signal         
	| signalset signal 
	;

signal: event action ID  
                  {
                    Component component = (Component)containerStack.peek();
                    Component target = (Component)instanceTable.get($3.sval);
                    
                    if (target == null)
                       System.out.println("Widget "+$3.sval+" not defined");
                    else {
                       MouseEventHandler handler = 
                           new MouseEventHandler($1.ival, $2.ival, target);
                       component.addMouseListener(handler);
                    }
                  }

event: ONCLICK           { $$.ival = MouseEventHandler.ONCLICK; }
	| ONPRESS        { $$.ival = MouseEventHandler.ONPRESS; }
	| ONRELEASE      { $$.ival = MouseEventHandler.ONRELEASE; }
	| ONENTER        { $$.ival = MouseEventHandler.ONENTER; }
	| ONLEAVE        { $$.ival = MouseEventHandler.ONLEAVE; }
	| ONTOGGLE       { $$.ival = MouseEventHandler.ONCLICK; }
	;

action: SENSITIZE            { $$.ival = MouseEventHandler.SENSITIZE; }
	| INSENSITIZE        { $$.ival = MouseEventHandler.INSENSITIZE; }
	| TOGGLESENSITIVITY  { $$.ival = MouseEventHandler.TOGGLESENSITIVITY; }
	| SHOW               { $$.ival = MouseEventHandler.SHOW; }
	| HIDE               { $$.ival = MouseEventHandler.HIDE; }
	| TOGGLEVISIBILITY   { $$.ival = MouseEventHandler.TOGGLEVISIBILITY; }
	| SUBMITFORM         { $$.ival = MouseEventHandler.SENSITIZE; }
	| RESETFORM          { $$.ival = MouseEventHandler.SENSITIZE; }
	| CANCELFORM         { $$.ival = MouseEventHandler.SENSITIZE; }
	;

 
tooltip: /* empty */       { $$ = null; }
	| TOOLTIP qstring  { $$.sval = $2.sval; }

notebookblock: notebook optid OBRACE tabposition pages CBRACE
                     { 
                       containerStack.pop();
                       $$.obj = $1.obj;
                     }

notebook: NOTEBOOK      {
                          JTabbedPane tabs = new JTabbedPane();
                          containerStack.push(tabs);
                          $$.obj = tabs;
                        }

pages: pageblock           
	| pages pageblock 

pageblock: PAGE OBRACE title tooltip container CBRACE
          { 
            JTabbedPane tabs = (JTabbedPane)containerStack.peek();
            Container container = (Container)$5.obj;
            if ($4 != null) 
               tabs.addTab($3.sval, null, container, $4.sval);
            else
               tabs.addTab($3.sval, null, container);
          }

container: boxblock      { $$.obj = $1.obj; }
	| notebookblock  { $$.obj = $1.obj; }
	| frameblock     { $$.obj = $1.obj; }

tabposition: /* empty */      { }
	| TABPOSITION TOP     { }
	| TABPOSITION BOTTOM  { }
	| TABPOSITION LEFT    { }
	| TABPOSITION RIGHT   { }

radiogroupblock: radiogroup OBRACE radiobuttons CBRACE
                    { 
                      System.out.println("Found radiogroupblock");
                      containerStack.pop();
                      $$.obj = null;
                    }

radiogroup: RADIOGROUP     { 
                             System.out.println("Found radiogroup");
                             ButtonGroup radioGroup = new ButtonGroup();
                             containerStack.push(radioGroup);
                             $$.obj = radioGroup;
                           }

radiobuttons: 
	radiobuttonblock 
                 {
                   ButtonGroup radioGroup = (ButtonGroup)containerStack.peek(); 
                   JRadioButton radioButton = (JRadioButton)$1.obj; 
                   radioGroup.add(radioButton);
                 }
	| radiobuttons radiobuttonblock
                 {
                   ButtonGroup radioGroup = (ButtonGroup)containerStack.peek(); 
                   JRadioButton radioButton = (JRadioButton)$2.obj; 
                   radioGroup.add(radioButton);
                 }

radiobuttonblock: 
	radiobutton optid qstring  
             { 
               JRadioButton radioButton = (JRadioButton)containerStack.pop();  
               radioButton.setText($3.sval);
               ButtonGroup group = (ButtonGroup)containerStack.pop();
               Container container = (Container)containerStack.peek();
               containerStack.push(group);
               container.add(radioButton); 
               $$.obj = radioButton;
             }

	| radiobutton optid OBRACE borderwidth tooltip widgets CBRACE  
             {
               containerStack.pop();
               JRadioButton radioButton = (JRadioButton)$1.obj; 
               radioButton.setMargin(new Insets(0,9,0,0));
               ButtonGroup group = (ButtonGroup)containerStack.pop();
               Container container = (Container)containerStack.peek();
               containerStack.push(group);
               container.add(radioButton); 
               if ($5 != null)
                 radioButton.setToolTipText($5.sval);
               $$.obj = radioButton;
             }
	;
                            
radiobutton: RADIOBUTTON
                    {
                      JRadioButton radioButton = new JRadioButton();
                      containerStack.push(radioButton);
                      $$.obj = radioButton;
                    }
	;

listblock: list optid OBRACE listitems CBRACE
                {
                  JScrollPane scrollPane = (JScrollPane)containerStack.pop();
                  containerStack.pop();
                  $$.obj = scrollPane;
                }

list: LIST     {
                 DefaultListModel model = new DefaultListModel();
                 containerStack.push(model);
                 JList listBox = new JList();
                 listBox.setModel(model);
                 JScrollPane scrollPane = new JScrollPane(listBox);
                 containerStack.push(scrollPane);
                 $$.obj = model;
               }

listitems: listitemblock            
	| listitems listitemblock  

listitemblock: 
	LISTITEM qstring  
             {
               String label = new String($2.sval);
               JScrollPane scrollPane = (JScrollPane)containerStack.pop();
               DefaultListModel model = (DefaultListModel)containerStack.peek();
               containerStack.push(scrollPane);
               model.addElement(label);
             }

filechooser: FILECHOOSER qstring
             {
               JFileChooser chooser = new JFileChooser(new File($2.sval)); 
               $$.obj = chooser;
             }

dirchooser: DIRCHOOSER qstring
             {
               //JDirectoryPane dir = new JDirectoryPane(new File($2.sval)); 
               //$$.obj = dir;
             }

tableblock: table optid OBRACE borderwidth tableelements CBRACE
             {
                 JPanel table = (JPanel)$1.obj;
                 if ($2 != null) {
                   Border emptyBorder =
                     new EmptyBorder($2.ival,$2.ival,$2.ival,$2.ival);
                   table.setBorder(emptyBorder);
                 }
                 System.out.println("Found tableblock");
                 containerStack.pop();
                 containerStack.pop();
               }

table: TABLE NUMBER NUMBER
               {
                  System.out.println("Found table");
                  JPanel table = new JPanel();
                  GridBagLayout mgr = new GridBagLayout();
                  table.setLayout(mgr);
                  containerStack.push(mgr);
                  containerStack.push(table);
                  table.setOpaque(false);
                  $$.obj = table;
               }

tableelements: /* empty */
	| tabledata
	;

tabledata: tabledatumblock
	| tabledata tabledatumblock
	;

tabledatumblock: tabledatum OBRACE xpad ypad widgets CBRACE
               {
                  GridBagConstraints gbc = (GridBagConstraints)$1.obj;
                  //Seems like these two ought to work, but do not
                  //gbc.ipadx = $3.ival;
                  //gbc.ipady = $4.ival;
                  Insets inset = new Insets($4.ival,$3.ival,$4.ival,$3.ival);
                  gbc.insets = inset;
                  Component c = (Component)containerStack.pop();
                  GridBagLayout gbl = (GridBagLayout)containerStack.peek();
                  containerStack.push(c);
                  gbl.setConstraints((Component)$5.obj, gbc);
               }
                  
tabledatum: TABLEDATUM NUMBER NUMBER NUMBER NUMBER
               {
                  int left, right, top, bottom;
                  left = $2.ival;
                  right = $3.ival;
                  top = $4.ival;
                  bottom = $5.ival;
                  GridBagConstraints gbc = new GridBagConstraints();
                  gbc.gridx = left;
                  gbc.gridy = top;
                  gbc.gridwidth = right - left;
                  gbc.gridheight = bottom - top;
                  gbc.fill = GridBagConstraints.BOTH;
                  $$.obj = gbc;
               }


xpad: /* empty */           { $$ = new jmlparserval(0); }
	| XPADDING NUMBER   { $$.ival = $2.ival; }
                        
ypad: /* empty */           { $$ = new jmlparserval(0); }
	| YPADDING NUMBER   { $$.ival = $2.ival; }


comboboxblock: combobox OBRACE comboitems CBRACE
                      {
                         containerStack.pop();
                         $$.obj = $1.obj;
                      }

combobox: COMBOBOX ID
                      {   
                          JComboBox combobox = new JComboBox();
                          containerStack.push(combobox);
                          if (instanceTable.get($2.sval) == null)
                             instanceTable.put($2.sval, combobox);
                          else
                             System.out.println( "Duplicate ID " + $2.sval);
                          $$.obj = combobox;
                       }

comboitems: comboitem
	| comboitems comboitem
	;

comboitem: COMBOITEM qstring
                         { 
                            JComboBox cb = (JComboBox)containerStack.peek();
                            cb.addItem($2.sval);
                         }
                        

%%

String ins;
StringTokenizer st;
Yylex lexer;
Stack containerStack;
Hashtable instanceTable;

Font defaultFont = new Font("Dialog", Font.PLAIN, 12);
Font boldFont = new Font("Dialog", Font.BOLD, 12);
Font bigFont = new Font("Dialog", Font.PLAIN, 18);
Font bigBoldFont = new Font("Dialog", Font.BOLD, 18);
Font reallyBigFont = new Font("Dialog", Font.PLAIN, 18);
Font reallyBigBoldFont = new Font("Dialog", Font.BOLD, 24);
Font sansFont = new Font("SansSerif", Font.PLAIN, 12);


void yyerror(String s)
{
  System.out.println("par:"+s);
}

boolean newline;
int yylex()
{
  String s;
  int tok;
  Yytoken token;
    //System.out.print("yylex ");

    try
    {
      token = lexer.yylex();
    }
    catch (IOException e)
    {
      System.out.println(e.getMessage());
      token = null;
    }
    if (token != null)
    {
      //System.out.println("tok:"+token.m_text);
      tok = token.m_index;
      if (token.m_index == NUMBER)
      {
        Integer i;
        i = Integer.valueOf(token.m_text);
        yylval = new jmlparserval(i.intValue());
        return tok;
      }
      if ((token.m_index == OPERATOR) || 
          (token.m_index == NL) ||
          (token.m_index == CARET) ||
          (token.m_index == PAREN))
      {
        return token.m_text.charAt(0);
      }
      else
      {
        yylval = new jmlparserval(token.m_text);
        // System.out.println("Line: "+token.m_line);
        return tok;
      }
    }
    else
    {
      return 0;
    }
}

void dotest()
{
//  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
    System.out.println("Java Markup Language");
    containerStack = new Stack();
    lexer = new Yylex(System.in);
    instanceTable = new Hashtable();
    System.out.println("Done initializing Yylex");
    yyparse();
}

public static void main(String args[])
{
    jmlparser par = new jmlparser(false);
    par.dotest();
}
