/* JML - Java Markup Language
 * Copyright (C) 1998 Scott Dybiec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

import java.lang.System;

class Utility {
  public static void assert
    (
     boolean expr
     )
      { 
	if (false == expr) {
	  throw (new Error("Error: Assertion failed."));
	}
      }
  
  private static final String errorMsg[] = {
    "Error: Unmatched end-of-comment punctuation.",
    "Error: Unmatched start-of-comment punctuation.",
    "Error: Unclosed string.",
    "Error: Illegal character."
    };
  
  public static final int E_ENDCOMMENT = 0; 
  public static final int E_STARTCOMMENT = 1; 
  public static final int E_UNCLOSEDSTR = 2; 
  public static final int E_UNMATCHED = 3; 

  public static void error
    (
     int code
     )
      {
	System.out.println(errorMsg[code]);
      }
}

%%

%{
  private int comment_count = 0;
%} 
%line
%char

ws=[\ \t\b\012]+
comment=#.*
obrace=\{
cbrace=\}
qstring=\"[^\"\n]*[\"]
badqstring=\"[^\"\n]*$
id=[a-zA-Z][a-zA-Z0-9]*
number=[0-9]+
nl=\n
operator=[\+\-\*\/]
paren=[\(\)]


%% 

{ws}               { }
{comment}          { }
{qstring}          { String str =  yytext().substring(1,yytext().length() - 1);
	
                     Utility.assert(str.length() == yytext().length() - 2);
	             return (new Yytoken(jmlparser.QSTRING,str,yyline,
                                         yychar,yychar + str.length()));
                   }
{badqstring}       { String str =  yytext().substring(1,yytext().length());

	             Utility.error(Utility.E_UNCLOSEDSTR);
	             Utility.assert(str.length() == yytext().length() - 1);
	             return (new Yytoken(jmlparser.BADSTRING,str,yyline,
                                         yychar,yychar + str.length()));
                   }
{obrace}           { return (new Yytoken(jmlparser.OBRACE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
{cbrace}           { return (new Yytoken(jmlparser.CBRACE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

True               { return (new Yytoken(jmlparser.TRUE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
False              { return (new Yytoken(jmlparser.FALSE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Window             { return (new Yytoken(jmlparser.WINDOW, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Title              { return (new Yytoken(jmlparser.TITLE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

HorizontalBox      { return (new Yytoken(jmlparser.HORIZONTALBOX, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
VerticalBox        { return (new Yytoken(jmlparser.VERTICALBOX,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
BorderWidth        { return (new Yytoken(jmlparser.BORDERWIDTH,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
Homogeneous        { return (new Yytoken(jmlparser.HOMOGENEOUS,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
Spacing            { return (new Yytoken(jmlparser.SPACING, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

Frame              { return (new Yytoken(jmlparser.FRAME, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
FrameShadow        { return (new Yytoken(jmlparser.FRAMESHADOW,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
ShadowIn           { return (new Yytoken(jmlparser.SHADOW_IN, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
ShadowOut          { return (new Yytoken(jmlparser.SHADOW_OUT,yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
ShadowNone         { return (new Yytoken(jmlparser.SHADOW_NONE,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
ShadowEtchedIn     { return (new Yytoken(jmlparser.SHADOW_ETCHED_IN, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
ShadowEtchedOut    { return (new Yytoken(jmlparser.SHADOW_ETCHED_OUT, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

Button             { return (new Yytoken(jmlparser.BUTTON, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
ToggleButton       { return (new Yytoken(jmlparser.TOGGLEBUTTON, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
CheckButton        { return (new Yytoken(jmlparser.CHECKBUTTON,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
Label              { return (new Yytoken(jmlparser.LABEL, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

ToolTip            { return (new Yytoken(jmlparser.TOOLTIP, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Entry              { return (new Yytoken(jmlparser.ENTRY, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

NoteBook           { return (new Yytoken(jmlparser.NOTEBOOK, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Page               { return (new Yytoken(jmlparser.PAGE, yytext(), yyline,
                                         yychar, yychar + yytext().length())); }
TabPosition        { return (new Yytoken(jmlparser.TABPOSITION,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
Top                { return (new Yytoken(jmlparser.TOP, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Bottom             { return (new Yytoken(jmlparser.BOTTOM, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Left               { return (new Yytoken(jmlparser.LEFT, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Right              { return (new Yytoken(jmlparser.RIGHT, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Fill               { return (new Yytoken(jmlparser.FILL, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }


RadioGroup         { return (new Yytoken(jmlparser.RADIOGROUP,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
RadioButton        { return (new Yytoken(jmlparser.RADIOBUTTON, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

List               { return (new Yytoken(jmlparser.LIST, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
ListItem           { return (new Yytoken(jmlparser.LISTITEM, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

Widget             { return (new Yytoken(jmlparser.WIDGET, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }

OnClick            { return (new Yytoken(jmlparser.ONCLICK, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
OnPress            { return (new Yytoken(jmlparser.ONPRESS, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
OnRelease          { return (new Yytoken(jmlparser.ONRELEASE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
OnEnter            { return (new Yytoken(jmlparser.ONENTER, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
OnLeave            { return (new Yytoken(jmlparser.ONLEAVE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
OnToggle           { return (new Yytoken(jmlparser.ONTOGGLE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Sensitize          { return (new Yytoken(jmlparser.SENSITIZE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Insensitize        { return (new Yytoken(jmlparser.INSENSITIZE,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
ToggleSensitivity  { return (new Yytoken(jmlparser.TOGGLESENSITIVITY,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
Show               { return (new Yytoken(jmlparser.SHOW, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
Hide               { return (new Yytoken(jmlparser.HIDE, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
ToggleVisibility   { return (new Yytoken(jmlparser.TOGGLEVISIBILITY, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
SubmitForm         { return (new Yytoken(jmlparser.SUBMITFORM,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
ResetForm          { return (new Yytoken(jmlparser.RESETFORM, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
CancelForm         { return (new Yytoken(jmlparser.CANCELFORM,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
Justify            { return (new Yytoken(jmlparser.JUSTIFY,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
Center             { return (new Yytoken(jmlparser.CENTER,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
FileChooser        { return (new Yytoken(jmlparser.FILECHOOSER,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
DirectoryChooser   { return (new Yytoken(jmlparser.DIRCHOOSER,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
Table              { return (new Yytoken(jmlparser.TABLE,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
TableDatum         { return (new Yytoken(jmlparser.TABLEDATUM,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
XPadding           { return (new Yytoken(jmlparser.XPADDING,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
YPadding           { return (new Yytoken(jmlparser.YPADDING,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
ComboBox           { return (new Yytoken(jmlparser.COMBOBOX,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }
ComboItem          { return (new Yytoken(jmlparser.COMBOITEM,yytext(),yyline,
                                         yychar, yychar + yytext().length()));
                   }










{id}               { return (new Yytoken(jmlparser.ID, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
{number}           { return (new Yytoken(jmlparser.NUMBER, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
{nl}               { return (new Yytoken(jmlparser.NL, yytext(), yyline,
                                         yychar, yychar + yytext().length()));
                   }
.                  { System.out.println("Illegal character: <"+yytext()+">");
	             Utility.error(Utility.E_UNMATCHED);
                   }
